@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
+from functools import lru_cache
 
 import abc
 import clu.abstract
@@ -16,7 +17,8 @@ from clu.config.keymapview import KeyMapKeysView, KeyMapItemsView, KeyMapValuesV
 from clu.config.keymapview import NamespaceWalkerKeysView, NamespaceWalkerItemsView
 from clu.config.keymapview import NamespaceWalkerValuesView
 
-from clu.predicates import (isexpandable, iscontainer, isnotnone,
+from clu.predicates import (unwrap,
+                            isexpandable, iscontainer, isnotnone,
                             always, uncallable)
 
 from clu.typology import iterlen, ismapping
@@ -25,6 +27,8 @@ from clu.exporting import Exporter
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
 
+cache = lambda function: lru_cache(maxsize=128, typed=True)(function)
+
 # SUB-BASE AND ABSTRACT BASE CLASSES:
 
 @export
@@ -63,6 +67,10 @@ class FrozenKeyMapBase(collections.abc.Mapping,
     def __getitem__(self, nskey):
         ...
     
+    @abstract
+    def __hash__(self):
+        ...
+    
     def __reversed__(self):
         yield from reversed(tuple(self))
     
@@ -76,7 +84,8 @@ class FrozenKeyMapBase(collections.abc.Mapping,
 
 @export
 class KeyMapBase(FrozenKeyMapBase,
-                 collections.abc.MutableMapping):
+                 collections.abc.MutableMapping,
+                 clu.abstract.Unhashable):
     
     """ Abstract sub-base interface class for mutable namespaced mappings.
         
@@ -127,7 +136,7 @@ class FrozenKeyMap(FrozenKeyMapBase):
         return default
     
     def submap(self, *namespaces, unprefixed=False):
-        """ Return a standard dict containing only the namespaced items. """
+        """ Return a flattened dict containing only the namespaced items. """
         if unprefixed:
             return { nskey : self[nskey] for nskey in self if NAMESPACE_SEP not in nskey }
         if not namespaces:
@@ -162,7 +171,14 @@ class FrozenKeyMap(FrozenKeyMapBase):
             return self.submap(unprefixed=unprefixed).values()
         return KeyMapValuesView(self, *namespaces)
     
-    def namespaces(self):
+    def __hash__(self):
+        return hash(tuple(self.values()))
+    
+    @cache
+    def _get_namespace_foset(self):
+        return FlatOrderedSet(get_ns(nskey) for nskey in sorted(self) if NAMESPACE_SEP in nskey)
+    
+    def _namespaces(self): # pragma: no cover
         """ Iterate over all of the namespaces defined in the mapping.
             
             This is the generic implementation. It depends on “__iter__(…)”,
@@ -170,6 +186,12 @@ class FrozenKeyMap(FrozenKeyMapBase):
         """
         nss = { get_ns(nskey) for nskey in self if NAMESPACE_SEP in nskey }
         yield from sorted(nss)
+    
+    def namespaces(self):
+        yield from self._get_namespace_foset()
+    
+    def namespace_count(self):
+        return len(self._get_namespace_foset())
 
 @export
 class KeyMap(KeyMapBase, FrozenKeyMap):
@@ -202,9 +224,10 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         del self[nskey]
     
     def pop(self, key, *namespaces, default=NoDefault):
-        """ Pop a (possibly namespaced) value off the mapping
-            and eitehr return it or a default if it doesn’t
-            exist – raising a KeyError if no default is given.
+        """ Pop a (possibly namespaced) value off of the mapping
+            and return either that value, or a default value
+            if it doesn’t exist – raising a KeyError if no
+            default value is given.
         """
         value = self.get(key, *namespaces, default=default)
         if value == default or value is default:
@@ -212,6 +235,31 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         self.delete(key, *namespaces)
         return value
     
+    def popitem(self):
+        """ Pop a key/value pair off of the mapping and return
+            them as a tuple. KeyError is raised if the mapping
+            is empty.
+            
+            This may or may not be deterministc and/or random,
+            depending on the underlying implementation.
+        """
+        # Get an iterator for the KeyMap:
+        iterator = iter(self)
+        
+        # Advance the iterator once, getting a single (possibly
+        # namespaced) key from the KeyMap, and raisting a KeyError
+        # if there wasn’t anything found:
+        try:
+            nskey = next(iterator)
+        except StopIteration:
+            raise KeyError('popitem(): KeyMap is empty')
+        
+        # Get the corresponding value, delete it, and return
+        # the key/value pair as a tuple:
+        value = self[nskey]
+        del self[nskey]
+        return nskey, value
+    
     def clear(self, *namespaces, unprefixed=False):
         """ Remove all items from the mapping – either in totality,
             or only those matching a specific namespace.
@@ -222,10 +270,8 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
             return None
         if not namespaces:
             return super().clear()
-        prefix = prefix_for(*namespaces)
-        for nskey in self:
-            if nskey.startswith(prefix):
-                del self[nskey]
+        for nskey in self.submap(*namespaces).keys():
+            del self[nskey]
         return None
     
     def update(self, dictish=NoDefault, **updates):
@@ -236,10 +282,12 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         if dictish is not NoDefault:
             if hasattr(dictish, 'items'):
                 dictish = dictish.items()
-            for key, value in dictish:
-                self[key] = value
+            for nskey, value in dictish:
+                self[nskey] = value
         if updates:
             self.update(dictish=updates)
+    
+    _get_namespace_foset = lambda self: unwrap(super()._get_namespace_foset)(self)
 
 # INTERIM ABSTRACT BASE: NamespaceWalker
 
@@ -296,14 +344,11 @@ class NamespaceWalker(FrozenKeyMap):
     def flatten(self, cls=None):
         """ Dearticulate an articulated KeyMap instance into one that is flat. """
         if cls is None:
-            from clu.config.keymap import Flat
-            cls = Flat
-        out = cls()
-        for *namespaces, key, value in self.walk():
-            out.set(key, value, *namespaces)
-        return out
+            from clu.config.keymap import FrozenFlat
+            cls = FrozenFlat
+        return cls({ pack_ns(key, *namespaces) : value for *namespaces, key, value in self.walk() })
     
-    def namespaces(self):
+    def _namespaces(self): # pragma: no cover
         """ Iterate over all of the namespaces defined in the mapping. """
         nss = set()
         for *namespaces, key, value in self.walk():
@@ -311,6 +356,10 @@ class NamespaceWalker(FrozenKeyMap):
                 nss.add(concatenate_ns(*namespaces))
         yield from sorted(nss)
     
+    # @cache
+    def _get_namespace_foset(self):
+        return FlatOrderedSet(concatenate_ns(*ns) for *ns, _, _ in self.walk() if ns)
+    
     def keys(self, *namespaces, unprefixed=False):
         """ Return a namespaced view over either all keys in the mapping,
             or over only those keys in the mapping matching the specified
@@ -395,6 +444,11 @@ class FlatOrderedSet(collections.abc.Set,
     """
     __slots__ = ('things', 'predicate')
     
+    @classmethod
+    def _from_iterable(cls, iterable): # pragma: no cover
+        # Required by the “collections.abc.Set” API:
+        return cls(*iterable)
+    
     @classmethod
     def is_a(cls, instance):
         return isinstance(instance, (cls, FlatOrderedSet)) or \
@@ -449,7 +503,7 @@ class FlatOrderedSet(collections.abc.Set,
         return len(self.things) > 0
     
     def __hash__(self):
-        return hash(self.things) & hash(id(self.things))
+        return self._hash()
     
     def __eq__(self, other):
         if type(self).is_a(other):
