@@ -2,14 +2,11 @@
 from __future__ import print_function
 
 import clu.abstract
-import collections
-import collections.abc
 import copy
 import sys
 
 from clu.config.abc import FrozenKeyMap, KeyMap, NamespaceWalker
-from clu.config.ns import unpack_ns, pack_ns
-from clu.predicates import tuplize
+from clu.config.ns import concatenate_ns, unpack_ns, pack_ns
 from clu.typology import ismapping
 from clu.exporting import Exporter
 
@@ -18,13 +15,41 @@ export = exporter.decorator()
 
 # CONCRETE SUBCLASSES: FrozenFlat and Flat
 
+@export
+def flatwalk(mapping):
+    """ Iteratively walk a flat mapping with namespaced keys,
+        as found in a FrozenFlat or Flat instance.
+        
+        Based on https://stackoverflow.com/a/12507546/298171
+    """
+    for nskey in mapping.keys():
+        key, namespaces = unpack_ns(nskey)
+        yield namespaces + [key, mapping[nskey]]
+
+@export
+def articulate(mapping):
+    """ Articulate a flat mapping with namespaced keys into
+        one that is nested.
+    """
+    tree = {}
+    for *namespaces, key, value in flatwalk(mapping):
+        d = tree
+        for namespace in namespaces:
+            try:
+                d = d[namespace]
+            except KeyError:
+                d[namespace] = {}
+                d = d[namespace]
+        d[key] = value
+    return tree
+
 @export
 class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
                                clu.abstract.Cloneable):
     
     """ A concrete immutable – or frozen – KeyMap class with a flat internal topology. """
     
-    __slots__ = tuplize('dictionary')
+    __slots__ = 'dictionary'
     
     def __init__(self, dictionary=None, **updates):
         """ Initialize a flat KeyMap instance from a target dictionary.
@@ -48,10 +73,8 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
     def nestify(self, cls=None):
         """ Articulate a flattened KeyMap instance out into one that is nested. """
         if cls is None:
-            cls = Nested
-        out = cls()
-        out.update(self.dictionary)
-        return out
+            cls = FrozenNested
+        return cls(tree=articulate(self.dictionary))
     
     def __iter__(self):
         yield from self.dictionary
@@ -89,19 +112,17 @@ class Flat(FrozenFlat, KeyMap):
     def __delitem__(self, nskey):
         del self.dictionary[nskey]
 
-def DefaultTree(*args, **kwargs):
-    """ Initialize a recursive DefaultDict pseudo-tree. """
-    return collections.defaultdict(DefaultTree, *args, **kwargs)
-
-def dictify(tree):
-    """ Recursively convert a nested mapping to standard dicts. """
-    if ismapping(tree):
-        return { key : dictify(tree[key]) for key in tree }
-    return tree
+@export
+def dictify(treeish):
+    """ Recursively convert a possibly nested mapping to standard dicts. """
+    if ismapping(treeish):
+        return { key : dictify(value) for key, value in treeish.items() }
+    return treeish
 
 @export
 def mapwalk(mapping, pre=None):
     """ Iteratively walk a nested mapping.
+        
         Based on https://stackoverflow.com/a/12507546/298171
     """
     pre = pre and pre[:] or []
@@ -124,7 +145,7 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
         or, if you will, a nested – internal topology.
     """
     
-    __slots__ = tuplize('tree')
+    __slots__ = 'tree'
     
     def __init__(self, tree=None, **updates):
         """ Initialize an articulated (née “nested”) KeyMap instance from a
@@ -138,7 +159,7 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
             tree = getattr(tree, 'tree')
         elif hasattr(tree, 'nestify'):
             tree = getattr(tree.nestify(), 'tree')
-        self.tree = DefaultTree(tree or {})
+        self.tree = dictify(dict(tree or {}))
         if updates:
             self.tree.update(**updates)
     
@@ -147,15 +168,18 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
         yield from mapwalk(self.tree)
     
     def submap(self, *namespaces, unprefixed=False):
-        """ Return a standard dict containing only the namespaced items. """
-        # The “unprefixed” codepath is optimized here, the rest
-        # delegates up to the ancestor implementation:
+        """ Return a flattened dict containing only the namespaced items. """
         if unprefixed:
             return { key : value for key, value in self.tree.items() \
                                   if not ismapping(value) }
         if not namespaces:
             return dict(self)
-        return super().submap(*namespaces)
+        if not concatenate_ns(*namespaces) in self.namespaces():
+            return {}
+        d = self.tree
+        for namespace in namespaces:
+            d = d[namespace]
+        return { pack_ns(key, *namespaces) : value for key, value in d.items() }
     
     def inner_repr(self):
         return repr(self.tree)
@@ -163,6 +187,27 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
     def clone(self, deep=False, memo=None):
         copier = deep and copy.deepcopy or copy.copy
         return type(self)(tree=copier(self.tree))
+    
+    def __contains__(self, nskey):
+        key, namespaces = unpack_ns(nskey)
+        if not namespaces:
+            return key in self.tree
+        d = self.tree
+        for namespace in namespaces:
+            try:
+                d = d[namespace]
+            except KeyError:
+                return False
+        return key in d
+    
+    def __getitem__(self, nskey):
+        key, namespaces = unpack_ns(nskey)
+        if not namespaces:
+            return self.tree[key]
+        d = self.tree
+        for namespace in namespaces:
+            d = d[namespace]
+        return d[key]
 
 @export
 class Nested(FrozenNested, KeyMap):
@@ -184,7 +229,8 @@ class Nested(FrozenNested, KeyMap):
                 try:
                     d = d[namespace]
                 except KeyError:
-                    d = d[namespace] = type(d)()
+                    d[namespace] = {}
+                    d = d[namespace]
             d[key] = value
     
     def __delitem__(self, nskey):
@@ -288,6 +334,19 @@ def test():
         
         flat = FrozenFlat(flat_dict)
         nested = Nested(flat)
+        
+        print("FLAT:")
+        pprint(flat)
+        print()
+        
+        print("NESTED:")
+        pprint(nested)
+        print()
+        
+        print("NESTED FLATTEN:")
+        pprint(nested.flatten())
+        print()
+        
         assert nested.flatten() == flat
         assert nested.flatten().dictionary == flatdict()
         assert nested == flat
