@@ -25,10 +25,11 @@ def checkmanifest(session):
 @nox.parametrize('module', (
     nox.param('clu.constants',    id='consts'),
     nox.param('clu',              id='modules'),
+    nox.param('clu.importing',    id='importing'),
     nox.param('clu.version',      id='version'),
     nox.param('clu.scripts.repl', id='repl')))
 def checkmodule(session, module):
-    """ Check CLU consts, modules, REPL, or versioning """
+    """ Check CLU consts, modules, importing, REPL, or version """
     session.install("-r", "requirements/install.txt")
     if module == 'clu':
         session.install("-r", "requirements/nox/tests.txt")
@@ -40,16 +41,16 @@ def checkmodule(session, module):
 def pytest(session):
     """ Run CLU’s entire unit-test suite with `pytest` """
     session.env['MACOSX_DEPLOYMENT_TARGET']       = '10.14'
-    session.env['PYTEST_DISABLE_PLUGIN_AUTOLOAD'] = '1'
+    # session.env['PYTEST_DISABLE_PLUGIN_AUTOLOAD'] = '1'
     session.install("-r", "requirements/install.txt")
     session.install("-r", "requirements/nox/repl.txt")
-    session.run('pytest')
+    session.run('pytest', '--no-delete-temps')
 
 def parametrized_inline_tests():
-    from clu import all
-    for dotpath in all.inline_tests():
+    import clu.all
+    for dotpath in clu.all.inline_tests():
         yield nox.param(dotpath,
-                     id=dotpath.lstrip('clu').lstrip('.'))
+                     id=dotpath.removeprefix('clu.'))
 
 @nox.session
 @nox.parametrize('module', tuple(parametrized_inline_tests()))
@@ -63,7 +64,7 @@ def inline(session, module):
 @nox.session
 def codecov(session):
     """ Run `codecov`, updating CLU’s statistics on codecov.io """
-    from clu import all
+    import clu.all
     from clu.fs.filesystem import TemporaryName
     coveragefile = TemporaryName(prefix='coverage-',
                                  suffix='bin',
@@ -95,7 +96,7 @@ def codecov(session):
                     silent=True)
     
     # Run each inline-test function:
-    for modulename in all.inline_tests():
+    for modulename in clu.all.inline_tests():
         session.run('coverage',
                     'run', '--append', '-m', modulename,
                      silent=True)
