@@ -26,7 +26,7 @@ class TestImporting(object):
         assert hash(subscript) == hash(ModuleAlias(ProxyModule, Module))
         
         reprstring = "ModuleAlias(origin=‘<class-module “ProxyModule”>’, " \
-                     "specializer=’<class-module “clu.app.Module”>’)"
+                     "specializer=‘<class-module “clu.app.Module”>’)"
         assert repr(subscript).startswith(reprstring)
         
         unspecialized = ProxyModule[None]
@@ -36,6 +36,8 @@ class TestImporting(object):
         assert unspecialized.specializer is None
         assert unspecialized.__mro_entries__((object,)) == (ProxyModule,)
         assert unspecialized(object) == (ProxyModule,) # calls __mro_entries__(…)
+        assert unspecialized.__args__ == (ProxyModule,)
+        assert unspecialized.__origin__ == ProxyModule
         
         assert unspecialized == ModuleAlias(ProxyModule, None)
         assert unspecialized != ModuleAlias(Module, None)
@@ -43,7 +45,7 @@ class TestImporting(object):
         assert unspecialized != None
         
         reprstring = "ModuleAlias(origin=‘<class-module “ProxyModule”>’, " \
-                     "specializer=’None’)"
+                     "specializer=‘None’)"
         assert repr(unspecialized).startswith(reprstring)
         
         with pytest.raises(TypeError) as exc:
@@ -62,7 +64,7 @@ class TestImporting(object):
         assert repr(ModuleAlias) == reprstring
         
         reprstring = "ModuleAlias(origin=‘<class-module “ModuleBase”>’, " \
-                     "specializer=’<class-module “clu.app.Module”>’) @ 0x"
+                     "specializer=‘<class-module “clu.app.Module”>’) @ 0x"
         
         assert repr(ModuleBase[Module]).startswith(reprstring)
         assert repr(ModuleBase[Module][Module]).startswith(reprstring)
@@ -73,7 +75,7 @@ class TestImporting(object):
         assert repr(ModuleAlias(ModuleBase, None)[Module]).startswith(reprstring)
         
         reprstring = "ModuleAlias(origin=‘<class-module “clu.app.Module”>’, " \
-                     "specializer=’None’) @ 0x"
+                     "specializer=‘None’) @ 0x"
         
         assert repr(Module[None]).startswith(reprstring)
         assert repr(ModuleAlias[Module]).startswith(reprstring)
